package com.sms.alain.myserversms;

import android.app.Service;
import android.content.Intent;
import android.os.Handler;
import android.os.IBinder;
import android.telephony.SmsManager;
import android.util.Log;
import android.widget.Toast;

import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;

import java.io.IOException;


import fi.iki.elonen.NanoHTTPD;

public class ServeurSMSService extends Service {
    private MyHTTPD server;

    /**
     * Methode appelee au lancement du service.
     * Cree le socket d'ecoute
     * et affiche un Toast de prise en compte
     *
     * @return reponse de la superClasse
     */
    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        String ip = intent.getStringExtra("IP");
        String port = intent.getStringExtra("PORT");
        try {
            server = new MyHTTPD(Integer.parseInt(port));
            server.start();
        } catch (IOException e) {
            e.printStackTrace();
        }
        Toast.makeText(this, "Service activé", Toast.LENGTH_SHORT).show();
        return super.onStartCommand(intent, flags, startId);
    }

    /**
     * Methode appelee lors de la destruction du service.
     * Libere le socket si celui-ci existait
     * et affiche un Toast de prise en compte
     */
    @Override
    public void onDestroy() {
        if (server != null) {
            server.stop();
            Toast.makeText(this, "Service stoppé", Toast.LENGTH_SHORT).show();
        }
    }

    @Override
    public IBinder onBind(Intent intent) {
        throw new UnsupportedOperationException("Not yet implemented");
    }

    private class MyHTTPD extends NanoHTTPD {
        JSONdecodeur jsonDec = new JSONdecodeur();

        /**
         * Constructeur qui appelle la superClasse
         *
         * * @param int port d'ecoute du socket
         */
        public MyHTTPD(int port) throws IOException {
            super(port);
        }

        /**
         * Methode appelee apres start.
         * Cree le socket d'ecoute et attend le client
         *
         * @param IHTTPSession session
         * @return Response code de fin de connection 200 si ok
         */
        @Override
        public Response serve(IHTTPSession session) {

            byte[] messageDuClient = new byte[200];
            int tailleMessageDuClient=0;
            try {
                tailleMessageDuClient = session.getInputStream().read(messageDuClient);
                String jsonMsg = new String(messageDuClient);
                jsonMsg = jsonMsg.substring(0,tailleMessageDuClient);
                jsonDec.parser(jsonMsg);

                SmsManager smsManager = SmsManager.getDefault();  // envoi du SMS
                smsManager.sendTextMessage(jsonDec.getNumTel(), null, jsonDec.getTxtSMS(),
                        null, null);
                sendInfo();
            } catch (IOException e) {
                e.printStackTrace();
            }

            return newFixedLengthResponse(java.net.HttpURLConnection.HTTP_OK + "\n");
        }

        /**
         * Apres reception d'un SMS, cette methode diffuse
         * les informations via un intent
         *
         */
        private void sendInfo() {
            Intent intent = new Intent();
            intent.setAction(MainActivity.INTENT_NAME);
            intent.putExtra("text", jsonDec.getTxtSMS());
            intent.putExtra("number", jsonDec.getNumTel());
            sendBroadcast(intent);
        }
    }

    private class JSONdecodeur {

        private String numTel, txtSMS;

        /**
         * Methode d'analyse d'une String au format JSON
         * et qui extrait les valeurs des champs "number" et "text"
         *
         * @param String jsonString
         */
        void parser(String jsonString) {
            JSONParser jsonParser = new JSONParser();
            try {
                JSONObject jsonObject = (JSONObject) jsonParser.parse(jsonString);

                JSONObject jsonSMS = (JSONObject) jsonObject.get("SMS"); // retourne un JSONObject
                // qui est la valeur du champ SMS

                numTel = (String) jsonSMS.get("number");  // retourne la valeur du champ number
                txtSMS = (String) jsonSMS.get("text");    // retourne la valeur du champ text

            } catch (ParseException e) {
                e.printStackTrace();
            }
        }

        /**
         * Retourne le numero du SMS à envoyer, extrait du code JSON
         *
         * @return String numero du SMS
         */
        String getNumTel() {
            return numTel;
        }
        /**
         * Retourne le texte du SMS à envoyer, extrait du code JSON
         *
         * @return String message du SMS
         */
        String getTxtSMS() {
            return txtSMS;
        }
    }
}
